
%%                                                        READ STAG-YY 3.4

%    . calls f_readOther
%                                   initially written by Boris Kaus, <2010
%                                    modified by Paul Tackley, August 2010
%                               last modified by Fabio Crameri, 26.01.2017

%% NOTES
% Reads a STAGYY output file and transforms it into a MATLAB file
%
% Syntax:
%     For scalar fields:
%
%       [X_3D, Y_3D, Z_3D, DATA_3D] = ReadStag3D(fname_input, fname_number, {'viscosity','temperature'}, SWITCH, ioMode);
%
%   For vector fields:
%
%       [X_3D, Y_3D, Z_3D, VX_3D, VY_3D, VZ_3D, P_3D] = ReadStag3D(fname_input, fname_number, {'viscosity','temperature'}, SWITCH, ioMode);
%
%   or simply:
%
%       [READ] = ReadStag3D(...);
%
% use:   if ischar(X_3D); error(X_3D); end     to check if file is found.

function [varargout] = f_readStagYY(directory,fname_input,fname_number,Type,SWITCH,ioMode)

%% DEFAULTS
if ~exist('ioMode','var') || isempty(ioMode); ioMode = 1; end

%% CHECK WHICH GEODYNAMIC CODE TO READ
if ~isfield(SWITCH,'GeodynamicCode') || strcmp(SWITCH.GeodynamicCode,'StagYY')
    % use the function below
else %if strcmp(SWITCH.GeodynamicCode,'->CODENAME<-')
    [varargout] = f_readOther(directory,fname_input,fname_number,Type,SWITCH,ioMode);
    return
end

%% READ FILE
start_dir = pwd;
cd(directory);

FileFormat = 'n';           % native - default
% FileFormat = 'l';           % Little Endian
% FileFormat = 'b';           % Big    Endian

if fname_number<10000
    number_string = num2str(10000+fname_number);
    number_string(1)='0';
else
    number_string = num2str(10000+fname_number);
end

switch Type
    case {'Velocity', 'Horizontal Velocity', 'Radial Velocity', 'Pressure', 'Streamfunction'}
        fname       = [fname_input,'_vp',number_string];
        scalardata  = false;
    case 'Temperature'
        fname       = [fname_input,'_t',number_string];
        scalardata  = true;
    case 'Viscosity'
        fname       = [fname_input,'_eta',number_string];
        scalardata  = true;
    case 'Composition'
        fname       = [fname_input,'_c',number_string];
        scalardata  = true;
    case 'Melt fraction'
        fname       = [fname_input,'_f',number_string];
        scalardata  = true;
    case 'Stress'
        fname       = [fname_input,'_str',number_string];
        scalardata  = true;
    case {'Horizontal Princ. Stress', 'Radial Princ. Stress'}
        fname       = [fname_input,'_sx',number_string];
        scalardata  = false;
    case 'Strain Rate'
        fname       = [fname_input,'_ed',number_string];
        scalardata  = true;
    case 'Density'
        fname       = [fname_input,'_rho',number_string];
        scalardata  = true;
    case 'Air'
        fname       = [fname_input,'_air',number_string];
        scalardata  = true;
    case 'Primordial'
        fname       = [fname_input,'_prm',number_string];
        scalardata  = true;
    case 'Basalt'
        fname       = [fname_input,'_bs',number_string];
        scalardata  = true;
    case 'Harzburgite'
        fname       = [fname_input,'_hz',number_string];
        scalardata  = true;
    case 'Cont. Crust'
        fname       = [fname_input,'_cc',number_string];
        scalardata  = true;
    case 'Metal'
        fname       = [fname_input,'_mtl',number_string];
        scalardata  = true;
    case 'Water'
        fname       = [fname_input,'_wtr',number_string];
        scalardata  = true;
    case 'Dyn pressure'
        fname       = [fname_input,'_pd',number_string];
        scalardata  = true;
    case 'Topography self-grav'
        fname       = [fname_input,'_csg',number_string];
        scalardata  = true;
    case 'Crustal thickness'
        fname       = [fname_input,'_cr',number_string];
        scalardata  = true;
    case 'Heat Flux'
        fname       = [fname_input,'_hf',number_string];
        scalardata  = true;
    case 'Age'
        fname       = [fname_input,'_age',number_string];
        scalardata  = true;
    case 'Geoid'
        fname       = [fname_input,'_g',number_string];
        scalardata  = true;
    case 'Toroidal'
        fname       = [fname_input,'_to',number_string];
        scalardata  = false;
    case 'Poloidal'
        fname       = [fname_input,'_po',number_string];
        scalardata  = false;
    case 'Deformation Mechanism'
        fname       = [fname_input,'_defm',number_string];
        scalardata  = true;
    case 'zz-Stress Component'
        fname       = [fname_input,'_nstr',number_string];
        scalardata  = true;
    case 'Topo normal stress'
        fname       = [fname_input,'_nst',number_string];
        scalardata  = true;
    case 'Topography'
        fname       = [fname_input,'_cs',number_string];
        if ~exist(fname,'file')
            fname       = [fname_input,'_sc',number_string];  
            if exist(fname,'file'); disp('     -> _sc used for topography'); end
        end
        if ~exist(fname,'file')
            fname       = [fname_input,'_fstopo',number_string];
            if exist(fname,'file'); disp('     -> _fstopo used for topography'); end
        end
        if ~exist(fname,'file')
            fname       = [fname_input,'_cs',number_string]; %just to have the original name for error message
        end
        scalardata  = true;
    otherwise
        error(['Unknown property: ',Type])
end
if ~exist(fname,'file')
    % The file does not exist and we should stop processing data
    file_stem_now = pwd;
    %warning(['The file - ',file_stem_now,'/',fname,' - does not exist!'])
    cd(start_dir);
    for i=1:10
        varargout{i}    = ['The file - ',file_stem_now,'/',fname,' - does not exist!'];
    end
    
    return
end

if ~isfield(SWITCH,'Precision') || strcmp(SWITCH.Precision,'single')
    PrecisionString={'single' 'int32'};
elseif strcmp(SWITCH.Precision,'double')
    PrecisionString={'double' 'int64'};
else
    error('Precision unknown!')
end

if scalardata
    nval    =   1;      % temperature has only one value
else
    nval    =   4;      % assumed that we have a velocity-pressure file
end

fid         =   fopen(fname,'r',FileFormat);        % Open File

%==========================================================================
% READ HEADER
%==========================================================================
magic       = fread(fid,1,char(PrecisionString(1,2)));  	% Version
if (magic<100 && nval>1) || (magic>300 && nval==1)  % check #components
    error('wrong number of components in field')
end
magic       = mod(magic,100);
if magic>=9 && nval==4
    xyp = 1;     % extra ghost point in x & y direction
else
    xyp = 0;
end

nxtot       = fread(fid,1,char(PrecisionString(1,2)));     	% nx total
nytot       = fread(fid,1,char(PrecisionString(1,2)));  	% ny total
nztot       = fread(fid,1,char(PrecisionString(1,2)));      % nz total
nblocks     = fread(fid,1,char(PrecisionString(1,2)));    	% # of blocks
Aspect      = fread(fid,2,char(PrecisionString(1,1)));     	% Aspect ratio
nnx         = fread(fid,1,char(PrecisionString(1,2)));  	% Number of parallel subdomains
nny         = fread(fid,1,char(PrecisionString(1,2)));  	%  in the x,y,z and b directions
nnz         = fread(fid,1,char(PrecisionString(1,2)));      %
nnb         = fread(fid,1,char(PrecisionString(1,2)));  	%

nz2         = nztot*2 + 1;
zg          = fread(fid,nz2,char(PrecisionString(1,1)));  	% z-coordinates

% compute nx, ny, nz and nb PER CPU
nx          =   nxtot/nnx;
ny          =   nytot/nny;
nz          =   nztot/nnz;
nb          =   nblocks/nnb;
npi         =   (nx+xyp)*(ny+xyp)*nz*nb*nval;      % the number of values per 'read' block

rcmb      	= fread(fid,1,char(PrecisionString(1,1)));
istep      	= fread(fid,1,char(PrecisionString(1,2)));
time       	= fread(fid,1,char(PrecisionString(1,1)));
erupta_total= fread(fid,1,char(PrecisionString(1,1)));
botT_val 	= fread(fid,1,char(PrecisionString(1,1)));

x           = fread(fid,nxtot,char(PrecisionString(1,1)));      % x-coordinates
y           = fread(fid,nytot,char(PrecisionString(1,1)));      % y-coordinates
z           = fread(fid,nztot,char(PrecisionString(1,1)));      % z-coordinates

% read the parallel blocks
if scalardata
    DATA_3D = zeros(nxtot,nytot,nztot);
else
    scalefac= fread(fid,1,char(PrecisionString(1,1)));     	% scale factor
    VX_3D   = zeros(nxtot,nytot,nztot);                     %   Vx
    VY_3D   = zeros(nxtot,nytot,nztot);                     %   Vy
    VZ_3D   = zeros(nxtot,nytot,nztot);                     %   Vz
    P_3D    = zeros(nxtot,nytot,nztot);                     %   Pressure
end

for ibc=1:nnb       % loop over parallel subdomains
    for izc =1:nnz
        for iyc =1:nny
            for ixc =1:nnx
                data_CPU    =   fread(fid,npi,char(PrecisionString(1,1)));      % read the data for this CPU
                
                % Create a 3D matrix from these data
                if scalardata
                    data_CPU_3D = reshape(data_CPU, [nx ny nz nb]);
                else
                    data_CPU_3D = reshape(data_CPU*scalefac, [nval nx+xyp ny+xyp nz nb]);
                end
                
                % Add local 3D matrix to global matrix
                if scalardata
                    % Scalar data
                    DATA_3D((ixc-1)*nx + (1:nx), (iyc-1)*ny + (1:ny), (izc-1)*nz + (1:nz),(ibc-1)*nb + (1:nb)) = data_CPU_3D;
                else
                    % velocity-pressure data
                    VX_3D((ixc-1)*nx + (1:nx), (iyc-1)*ny +  (1:ny), (izc-1)*nz + (1:nz), (ibc-1)*nb + (1:nb)) = squeeze(data_CPU_3D(1,1:nx,1:ny,:,:));
                    VY_3D((ixc-1)*nx + (1:nx), (iyc-1)*ny +  (1:ny), (izc-1)*nz + (1:nz), (ibc-1)*nb + (1:nb)) = squeeze(data_CPU_3D(2,1:nx,1:ny,:,:));
                    VZ_3D((ixc-1)*nx + (1:nx), (iyc-1)*ny +  (1:ny), (izc-1)*nz + (1:nz), (ibc-1)*nb + (1:nb)) = squeeze(data_CPU_3D(3,1:nx,1:ny,:,:));
                    P_3D( (ixc-1)*nx + (1:nx), (iyc-1)*ny +  (1:ny), (izc-1)*nz + (1:nz), (ibc-1)*nb + (1:nb)) = squeeze(data_CPU_3D(4,1:nx,1:ny,:,:));
                end
            end
        end
    end
end

fclose(fid);                                % close file

[Y_3D, X_3D, Z_3D]  = meshgrid(y,x,z);

%======================================
% decrease size of data
decrease_filesize = false;
nr_saveX = 2;   %saves every 'nr_saveX' number in x-direction
nr_saveY = 2;   %                              in y-direction
nr_saveZ = 2;   %                              in z-direction

if decrease_filesize
    if nblocks==1
        % no ying-yang
        numx = size(X_3D,1); numy = size(X_3D,2); numz = size(X_3D,3);
        dummy = X_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); X_3D = dummy;
        dummy = Y_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); Y_3D = dummy;
        dummy = Z_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); Z_3D = dummy;
        if ~scalardata
            dummy = VX_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); VX_3D = dummy;
            dummy = VY_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); VY_3D = dummy;
            dummy = VZ_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); VZ_3D = dummy;
            dummy = P_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); P_3D = dummy;
        else
            dummy = DATA_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz); DATA_3D = dummy;
        end
    else
        % ying-yang grid
        numx = size(X_3D,1); numy = size(X_3D,2); numz = size(X_3D,3);
        dummy = X_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); X_3D = dummy;
        dummy = Y_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); Y_3D = dummy;
        dummy = Z_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); Z_3D = dummy;
        if ~scalardata
            dummy = VX_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); VX_3D = dummy;
            dummy = VY_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); VY_3D = dummy;
            dummy = VZ_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); VZ_3D = dummy;
            dummy = P_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); P_3D = dummy;
        else
            dummy = DATA_3D(1:nr_saveX:numx,1:nr_saveY:numy,1:nr_saveZ:numz,:); DATA_3D = dummy;
        end
    end
    disp(['...saved every ',num2str(nr_saveX),'. x-grid point, ',num2str(nr_saveY),'. y-grid point, ',num2str(nr_saveZ),'. z-grid point'])
end
%======================================

%======================================
%% MIRROR DATA HORIZONTALLY
% SWITCH.flipDataHorizontally = true;
if isfield(SWITCH,'flipDataHorizontally') && SWITCH.flipDataHorizontally
    if nxtot==1 || nytot==1 %2-D geometry
        if logical(0) %flip coordinates
            %         X_3D        = max(X_3D(:))+min(X_3D(:))-X_3D;
            %         Y_3D        = max(X_3D(:))+min(X_3D(:))-X_3D;
            X_3D        = flipud(X_3D);
            Y_3D        = flipud(Y_3D);
        else %flip data
            if scalardata
                DATA_3D = flipud(DATA_3D);
            else
                VX_3D 	= flipud(VX_3D);
                VY_3D  	= flipud(VY_3D);
                VZ_3D   = flipud(VZ_3D);
                P_3D    = flipud(P_3D);
            end
        end
    end
end
%======================================


%% OUTPUT
if ioMode==1
    % prepare output data
    varargout{1} = nblocks;
    varargout{2} = X_3D;
    varargout{3} = Y_3D;
    varargout{4} = Z_3D;
    if nblocks==1
        % no yin-yang
        if ~scalardata
            varargout{5} = VX_3D;
            varargout{6} = VY_3D;
            varargout{7} = VZ_3D;
            varargout{8} = P_3D;
            varargout{9} = time;
            varargout{10} = rcmb;
            varargout{11} = fname;
            varargout{12} = Aspect;
        else
            varargout{5} = DATA_3D;
            varargout{6} = time;
            varargout{7} = rcmb;
            varargout{8} = fname;
            varargout{9} = Aspect;
        end
    else
        % yin-yang grid
        if ~scalardata
            varargout{5} = VX_3D(:,:,:,1);
            varargout{6} = VY_3D(:,:,:,1);
            varargout{7} = VZ_3D(:,:,:,1);
            varargout{8} = P_3D (:,:,:,1);
            
            varargout{9}  = VX_3D(:,:,:,2);
            varargout{10} = VY_3D(:,:,:,2);
            varargout{11} = VZ_3D(:,:,:,2);
            varargout{12} = P_3D (:,:,:,2);
            
            varargout{13} = time;
            varargout{14} = rcmb;
            varargout{15} = fname;
            varargout{16} = Aspect;
        else
            varargout{5} = DATA_3D(:,:,:,1);
            varargout{6} = DATA_3D(:,:,:,2);
            varargout{7} = time;
            varargout{8} = rcmb;
            varargout{9} = fname;
            varargout{10} = Aspect;
        end
    end
    
elseif ioMode==2 % * THIS VERSION NEEDS LESS DATA TRANSFER *
    % prepare output data
    READ.nb         = nblocks;
    READ.X_3D       = X_3D;
    READ.Y_3D       = Y_3D;
    READ.Z_3D       = Z_3D;
    READ.time       = time;
    READ.rcmb       = rcmb;
    READ.fname     	= fname;
    READ.Aspect    	= Aspect;
    if nblocks==1
        % no yin-yang
        if ~scalardata
            READ.VX_3D    	= VX_3D;
            READ.VY_3D   	= VY_3D;
            READ.VZ_3D   	= VZ_3D;
            READ.P_3D       = P_3D;
        else
            READ.VAR_3D   	= DATA_3D;
        end
    else
        % yin-yang grid
        if ~scalardata
            READ.VX_3Dyin   = VX_3D(:,:,:,1);
            READ.VY_3Dyin   = VY_3D(:,:,:,1);
            READ.VZ_3Dyin   = VZ_3D(:,:,:,1);
            READ.P_3Dyin    = P_3D(:,:,:,1);
            
            READ.VX_3Dyang	= VX_3D(:,:,:,2);
            READ.VY_3Dyang	= VY_3D(:,:,:,2);
            READ.VZ_3Dyang 	= VZ_3D(:,:,:,2);
            READ.P_3Dyang  	= P_3D(:,:,:,2);
        else
            READ.VAR_3Dyin  	= DATA_3D(:,:,:,1);
            READ.VAR_3Dyang   	= DATA_3D(:,:,:,2);
        end
    end
    varargout{1} = READ;
end

cd(start_dir);



